import json


class ApiError(Exception):
    """Represents an exception returned by the remote API."""
    def __init__(self, message: str, code: int = None):
        self.message = message
        self.code = code

    def __str__(self):
        if isinstance(self.message, str):
            return self.message
        else:
            return json.dumps(self.message)


class SberAcquiringException(Exception):
    def __init__(self, message: str, code: int = None):
        self.message = message
        self.code = code


class ActionException(SberAcquiringException):
    pass


class BadCredentialsException(SberAcquiringException):
    pass


class BadRequestException(SberAcquiringException):
    pass


class BadResponseException(SberAcquiringException):
    pass


class NetworkException(SberAcquiringException):
    pass


class InvalidRequestArguments(SberAcquiringException):
    pass



import logging
import requests

from .exceptions import ApiError, BadCredentialsException


class Client(object):
    """Performs requests to the SBER API."""

    URL = 'https://3dsec.sberbank.ru/payment/rest/'

    def __init__(self, username: str = None, password: str = None, token: str = None):
        """
        :param username: SBER API username
        :param password: SBER API password
        """

        if username and password:
            self.username = username
            self.password = password
            self.params = {
                'userName': self.username,
                'password': self.password,
            }
        elif token:
            self.token = token
            self.params = {
                'token': self.token,
            }
        else:
            raise BadCredentialsException('Авторизация через логин/пароль или токен, выберите что-то одно')

        self.default_headers = {
            'Accept': 'application/json',
            'Accept-Encoding': 'gzip,deflate,sdch',
            'Cache-Control': 'no-cache',
            'Content-Type': 'application/json',
        }
        self.logger = logging.getLogger('sber')

    def _get(self, url: str, params: dict = None):
        resp = requests.get(url, headers=self.default_headers, params=params)
        if not resp.ok:
            raise ApiError(resp.status_code, resp.text)
        return resp.json()

    def _post(self, url: str, params: dict = None):
        resp = requests.post(url, headers=self.default_headers, params=params)
        if not resp.ok:
            raise ApiError(resp.status_code, resp.text)
        return resp.json()

    def register_order(self, order_number: str, amount: int, return_url: str, **kwargs: dict):
        """
        Запрос регистрации заказа (register.do)
        https://securepayments.sberbank.ru/wiki/doku.php/integration:api:rest:requests:register

        :param order_number: Номер (идентификатор) заказа в системе магазина, уникален
                             для каждого магазина в пределах системы. Если номер заказа
                             генерируется на стороне платёжного шлюза, этот параметр
                             передавать необязательно.
        :param amount: Сумма возврата в минимальных единицах валюты.
        :param return_url: Адрес, на который требуется перенаправить пользователя в
                           случае успешной оплаты. Адрес должен быть указан полностью,
                           включая используемый протокол.
        :param kwargs: Необязательные данные.
        """
        url = f"{self.URL}register.do"
        params = {
            'orderNumber': order_number,
            'amount': amount,
            'returnUrl': return_url
        }
        # py3.9: self.params |= params

        return self._post(url, params={**self.params, **params})

    def register_order_pre_auth(self, order_number: str, amount: str, return_url: str, **kwargs: dict):
        """
        Запрос регистрации заказа с предавторизацией (registerPreAuth.do)
        https://securepayments.sberbank.ru/wiki/doku.php/integration:api:rest:requests:registerpreauth

        :param order_number: Номер (идентификатор) заказа в системе магазина, уникален
                             для каждого магазина в пределах системы. Если номер заказа
                             генерируется на стороне платёжного шлюза, этот параметр
                             передавать необязательно.
        :param amount: Сумма возврата в минимальных единицах валюты.
        :param return_url: Адрес, на который требуется перенаправить пользователя в
                           случае успешной оплаты. Адрес должен быть указан полностью,
                           включая используемый протокол.
        :param kwargs: Необязательные данные.
        """
        url = f"{self.URL}registerPreAuth.do"
        params = {
            'orderNumber': order_number,
            'amount': amount,
            'returnUrl': return_url
        }
        if kwargs.get('description'):
            params['description'] = kwargs.get('description')
        # py3.9: self.params |= params

        return self._post(url, params={**self.params, **params})



from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()
long_description = (here / 'README.rst').read_text(encoding='utf-8')

setup(
    name='sber-payments',
    version='0.1.0',
    author='Anton Popov',
    author_email='anton@jokerinteractive.ru',
    description='SBER Payments Python Client Library',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    url='https://github.com/jokerinteractive/sber-payments',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Office/Business',
        'Topic :: Office/Business :: Financial',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    package_dir={'': 'src'},
    packages=find_packages(where='src'),
    python_requires='>=3.6',
    project_urls={
        'Bug Reports': 'https://github.com/jokerinteractive/sber-payments/issues',
        'Funding': 'https://donate.pypi.org',
        'Say Thanks!': 'http://saythanks.io/to/example',
        'Source': 'https://github.com/jokerinteractive/sber-payments/',
    },
    install_requires=['requests'], 
)